// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef EVENTVIEWS_VERSION_H
#define EVENTVIEWS_VERSION_H

#define EVENTVIEWS_VERSION_STRING "6.7.3"
#define EVENTVIEWS_VERSION_MAJOR 6
#define EVENTVIEWS_VERSION_MINOR 7
#define EVENTVIEWS_VERSION_PATCH 3
#define EVENTVIEWS_VERSION ((6<<16)|(7<<8)|(3))

#endif
